/* Light.css */

@import '../../../base.css';

/*******************************************************************************
 * improve hard to read hover text
 */
.DOWN, .UNREACHABLE, .CRITICAL {
  A.link {
    @apply hover:text-blue-200;
  }
}

/*******************************************************************************
 * login page
 */
BODY.loginpage {
  @apply bg-gradient-to-t from-gray-200 via-slate-300 to-gray-200;

  DIV.loginmask {
    @apply bg-slate-100;

    BUTTON, A.button {
      @apply text-white bg-slate-500 hover:bg-slate-600;
    }

    HR {
      @apply border-gray-500;
    }

    DIV.spinner {
      @apply border-white border-r-transparent;
    }
  }
}
