DIV#editor {
  position: relative;
}

TD.dataVar DIV#editor {
  font-weight: normal;
  font-size: 11px;
}
.ace_comment {
  color: #006600 !important;
}
/* define host */
.ace_keyword, .ace_constant.ace_language {
  color: #871F78 !important;
}
/* macros $USER1$ */
.ace_keyword.ace_operator {
  color: #8B0000 !important;
}
/* object attributes: host_name */
.ace_support.ace_function {
  color: #23238E !important;
}

.dark .ace_comment {
  color: #00b600 !important;
}
/* define host */
.dark .ace_keyword, .ace_constant.ace_language {
  color: #ffc4f6 !important;
}
/* macros $USER1$ */
.dark .ace_keyword.ace_operator {
  color: #cf0000 !important;
}
/* object attributes: host_name */
.dark .ace_support.ace_function {
  color: #6767ff !important;
}

/* object browser */
/* icons layout */
DIV.layout_icons.browser_file_item {
  height: 65px;
  width: 100px;
  float: left;
  margin: 5px;
  z-index: 10;
  -moz-user-select: none;
  position: relative;
}
DIV.layout_icons.browser_file_item IMG {
  -moz-user-select: none;
  margin-left: 27px;
}
DIV.layout_icons DIV.browser_file_shorttitle {
  white-space: nowrap;
  -moz-user-select: none;
  position: absolute;
  bottom: 0px;
  width: 100%;
  height: 18px;
  text-align: center;
  overflow: visible;
}
DIV.layout_icons DIV.browser_file_longtitle {
  display: none;
}
DIV.layout_icons DIV.browser_file_type {
  display: none;
}
DIV.layout_icons DIV.browser_file_template {
  display: none;
}
DIV.layout_icons DIV.browser_file_filename {
  display: none;
}
DIV.layout_icons DIV.browser_file_linenr {
  display: none;
}
DIV.layout_icons DIV.browser_file_comments {
  display: none;
}
DIV.layout_icons.browser_file_menu {
  display: none;
}

/* table layout */
DIV.layout_table.browser_file_menu {
  font-weight: bold;
  height: 20px;
  width: 880px;
}
DIV.layout_table.browser_file_item {
  margin-top: 2px;
  -moz-user-select: none;
  height: 20px;
  width: 880px;
  overflow: hidden;
}
DIV.layout_table.browser_file_item IMG {
  width: 18px;
  height: 18px;
  float: left;
  clear: both;
}
DIV.layout_table DIV.browser_file_image {
  width: 18px;
  float: left;
  clear: both;
}
DIV.layout_table DIV.browser_file_shorttitle {
  display: none;
}
DIV.layout_table DIV.browser_file_longtitle {
  width: 250px;
  height: 20px;
  float: left;
  margin-left: 5px;
  overflow: hidden;
}
DIV.layout_table DIV.browser_file_type {
  width: 75px;
  height: 20px;
  overflow: hidden;
  float: left;
}
DIV.layout_table DIV.browser_file_template {
  width: 70px;
  height: 20px;
  float: left;
  text-align: center;
}
DIV.layout_table DIV.browser_file_filename {
  width: 290px;
  height: 20px;
  overflow: hidden;
  float: left;
}
DIV.layout_table DIV.browser_file_linenr {
  width: 30px;
  height: 20px;
  overflow: hidden;
  float: left;
  text-align: right;
}
DIV.layout_table DIV.browser_file_comments {
  margin-left: 5px;
  width: 130px;
  height: 20px;
  overflow: hidden;
  float: left;
}

/* make expanded tree not overlap */
DIV#container > UL {
  height: 100%;
  overflow-x: auto;
}

/* objects should always be open */
LI#o_objects > INS {
  display: none;
}
LI#o_objects > UL {
  margin-left: -17px;
}
