DIV.legend TABLE {
    width: auto !important;
}

TD.legendColorBox > DIV {
    border: none !important;
}

#raw_top {
    background-color: inherit !important;
    color: inherit !important;
}

#raw_top TD, #raw_top TH {
    background-color: inherit !important;
    color: inherit !important;
}
#raw_top TD {
    white-space: nowrap;
    text-align: center;
}
#raw_top TH:nth-child(12), #raw_top TD:nth-child(12) {
    text-align: left;
}
#raw_top TD:nth-child(1) {
    text-align: right;
}

.axis_x, .axis_y, .legendLabel {
    color: var(--text-body) !important;
}