ul {
    list-style-type: none;
}

.elementTitle {
    font-weight: bold;
    font-size: 13px;
}
.elementTitle2 {
    font-weight: bold;
    font-size: 12px;
}
.elementTitle3 {
    font-size: 11px;
}

.void {
    background-color: none !important;
}
