DIV.bp_zoom_container .bp_container_zoom {
    display: none;
}
DIV.bp_zoom_container:hover .bp_container_zoom {
    display: inherit;
}

DIV.bp_node.OK.testmode {
    background-image:url('images/testmode_ok.png');
}
DIV.bp_node.WARNING.testmode {
    background-image:url('images/testmode_warning.png');
}
DIV.bp_node.CRITICAL.testmode {
    background-image:url('images/testmode_critical.png');
}
DIV.bp_node.UNKNOWN.testmode {
    background-image:url('images/testmode_unknown.png');
}

DIV.bp_label {
    font-weight: bold;
    line-height: 11px;
    font-size: 11px;
    width: 120px;
    height: 20px;
    display: table;
    position: relative;
    top: 2px;
}
SPAN.bp_label {
    display: table-cell;
    vertical-align: middle;
}

.bp_sublabel {
    font-size: 9px;
    width: 120px;
    height: 18px;
    overflow: hidden;
}

.bp_node_edit_icon {
    position: absolute;
    top: 0px;
    right: -1px;
    display: none;
}
.bp_node:hover .bp_node_edit_icon {
    display: inherit;
}

.bp_node_bp_ref_icon, .bp_node_link_icon {
    position: absolute;
    bottom: -2px;
    right: -1px;
    display: none;
}
.bp_node:hover .bp_node_bp_ref_icon, .bp_node:hover .bp_node_link_icon, .bp_node:hover .bp_node_add_icon {
    display: inherit !important;
}

.bp_node:hover {
    box-shadow: 0 0 5px rgba(0, 150, 255, 1);
    -webkit-box-shadow: 0 0 5px rgba(0, 150, 255, 1);
    -moz-box-shadow: 0 0 5px rgba(0, 150, 255, 1);
    border:1px solid rgba(0, 150, 255, 0.8);
}

.bp_node_active {
    box-shadow: 0 0 10px rgba(96, 50, 200, 1) !important;
    -webkit-box-shadow: 0 0 10px rgba(96, 50, 200, 1) !important;
    -moz-box-shadow: 0 0 10px rgba(96, 50, 200, 1) !important;
    border:1px solid rgba(96, 50, 200, 0.8) !important;
}

.bp_vedge {
    border-left: 2px solid grey;
    position: absolute;
    width: 1px;
}
.bp_vedge_hover {
    border-left: 2px solid #0096FF;
    z-index: 1;
    box-shadow: 0 0 5px rgba(0, 150, 255, 1);
    -webkit-box-shadow: 0 0 5px rgba(0, 150, 255, 1);
    -moz-box-shadow: 0 0 5px rgba(0, 150, 255, 1);
}

.bp_hedge {
    border-top: 2px solid grey;
    position: absolute;
    height: 1px;
}
.bp_hedge_hover {
    border-top: 2px solid #0096FF;
    z-index: 1;
    box-shadow: 0 0 5px rgba(0, 150, 255, 1);
    -webkit-box-shadow: 0 0 5px rgba(0, 150, 255, 1);
    -moz-box-shadow: 0 0 5px rgba(0, 150, 255, 1);
}

/* additional styles when frames in panorama view */
BODY.iframed {
    background: white;
}
BODY.iframed DIV.bp_zoom_container {
    border: 0;
}
HTML.transparent BODY {
    background: transparent;
}
HTML.transparent DIV.bp_zoom_container {
    background-color: transparent;
}

#bp_edit_node_form INPUT.active {
    border: 1px solid var(--border-active);
}